"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const encoder_1 = require("@mesg/api/lib/util/encoder");
const grpc_1 = require("@mesg/api/lib/util/grpc");
const api_1 = __importDefault(require("@mesg/api"));
const resolve_1 = require("@mesg/api/lib/util/resolve");
const types_1 = require("@mesg/api/lib/types");
const defaultEndpoint = 'localhost:50052';
class Application {
    constructor(_api) {
        this.api = _api || new api_1.default(defaultEndpoint);
    }
    decodeData(data) {
        return encoder_1.decode(data);
    }
    encodeData(data) {
        return encoder_1.encode(data);
    }
    resolve(sid) {
        return __awaiter(this, void 0, void 0, function* () {
            return resolve_1.resolveSID(this.api, sid);
        });
    }
    listenEvent(request) {
        return this.api.event.stream(request);
    }
    listenResult(request) {
        return this.api.execution.stream({
            filter: Object.assign(Object.assign({}, (request.filter || {})), { statuses: [
                    types_1.ExecutionStatus.COMPLETED,
                    types_1.ExecutionStatus.FAILED,
                ] })
        });
    }
    executeTask(request) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.api.execution.create(request);
        });
    }
    executeTaskAndWaitResult(request) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const tag = uuid_1.v4();
                const stream = this.listenResult({
                    filter: {
                        executorHash: request.executorHash,
                        tags: [tag]
                    }
                })
                    .on('metadata', (metadata) => {
                    const err = grpc_1.checkStreamReady(metadata);
                    if (err == grpc_1.errNoStatus)
                        return;
                    if (err) {
                        stream.destroy(err);
                        return;
                    }
                    this.executeTask(Object.assign(Object.assign({}, request), { tags: [...(request.tags || []), tag] })).catch((err) => stream.destroy(err));
                })
                    .on('data', (result) => {
                    stream.cancel();
                    result.error ? reject(new Error(result.error)) : resolve(result);
                })
                    .on('error', (err) => {
                    stream.cancel();
                    reject(err);
                });
            });
        });
    }
}
exports.default = Application;
