"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const tape_1 = __importDefault(require("tape"));
const sinon = __importStar(require("sinon"));
const isUUID = __importStar(require("is-uuid"));
const _1 = __importDefault(require("."));
const mock_1 = __importStar(require("@mesg/api/lib/mock"));
tape_1.default('listenEvent() should listen for events and return a stream', (t) => {
    t.plan(2);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(api.event, 'stream');
    application.listenEvent({ filter: { instanceHash: Buffer.from('1'), key: '2' } });
    const req = spy.getCall(0).args[0];
    t.ok(Buffer.from('1').equals(req.filter.instanceHash));
    t.equal(req.filter.key, '2');
    spy.restore();
});
tape_1.default('listenResult() should listen for results and return a stream', (t) => {
    t.plan(2);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(api.execution, 'stream');
    application.listenResult({
        filter: {
            instanceHash: Buffer.from('1'),
            statuses: [mock_1.ExecutionStatus.COMPLETED]
        }
    });
    const req = spy.getCall(0).args[0];
    t.ok(Buffer.from('1').equals(req.filter.instanceHash));
    t.equal(req.filter.statuses[0], mock_1.ExecutionStatus.COMPLETED);
    spy.restore();
});
tape_1.default('executeTask() should execute a task', (t) => {
    t.plan(4);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(api.execution, 'create');
    application.executeTask({
        executorHash: Buffer.from('1'),
        taskKey: '2',
        inputs: application.encodeData({ foo: 'bar' }),
        tags: ['4', '5']
    });
    const req = spy.getCall(0).args[0];
    t.ok(Buffer.from('1').equals(req.executorHash));
    t.equal(req.taskKey, '2');
    t.equal(application.decodeData(req.inputs).foo, 'bar');
    t.same(req.tags, ['4', '5']);
    spy.restore();
});
tape_1.default('executeTask() should resolve promise with reply', (t) => {
    t.plan(1);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const reply = { hash: Buffer.from('1') };
    const stub = sinon.stub(api.execution, 'create').callsFake(res => Promise.resolve(reply));
    application.executeTask({
        executorHash: Buffer.from('2'),
        taskKey: '3',
        inputs: {}
    }).then(reply => t.ok(Buffer.from('1').equals(reply.hash)));
    stub.restore();
});
tape_1.default('executeTask() should reject promise with err', (t) => {
    t.plan(1);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const stub = sinon.stub(api.execution, 'create').callsFake(() => Promise.reject(new Error('1')));
    application.executeTask({
        executorHash: Buffer.from('2'),
        taskKey: '3',
        inputs: {}
    }).catch((err) => t.equal(err.message, '1'));
    stub.restore();
});
tape_1.default('executeTaskAndWaitResult() should listen for results', (t) => {
    t.plan(1);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(api.execution, 'stream');
    application.executeTaskAndWaitResult({ executorHash: Buffer.from('1'), taskKey: '2', inputs: {} });
    const req = spy.getCall(0).args[0];
    t.ok(Buffer.from('1').equals(req.filter.executorHash));
    spy.restore();
});
tape_1.default('executeTaskAndWaitResult() should reject and cancel result stream on `error` event', (t) => {
    t.plan(2);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(mock_1.streams.execution, 'cancel');
    application.executeTaskAndWaitResult({ executorHash: Buffer.from('2'), taskKey: '3', inputs: {} })
        .catch((err) => t.equal(err, '1'));
    mock_1.streams.execution.emit('error', '1');
    t.ok(spy.called);
    spy.restore();
});
tape_1.default('executeTaskAndWaitResult() should resolve and cancel result stream on first `data` event', (t) => {
    t.plan(2);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(mock_1.streams.execution, 'cancel');
    application.executeTaskAndWaitResult({ executorHash: Buffer.from('2'), taskKey: '3', inputs: {} })
        .then((result) => t.equal(result.hash, '2'));
    mock_1.streams.execution.emit('data', { hash: '2' });
    t.ok(spy.called);
    spy.restore();
});
tape_1.default('executeTaskAndWaitResult() should execute task', (t) => {
    t.plan(4);
    const api = mock_1.default('');
    const application = new _1.default(api);
    const spy = sinon.spy(api.execution, 'create');
    application.executeTaskAndWaitResult({ executorHash: Buffer.from('2'), taskKey: '3', inputs: application.encodeData({ foo: 'bar' }) });
    mock_1.streams.execution.emit('metadata', { get() { return ['ready']; } });
    const req = spy.getCall(0).args[0];
    t.ok(Buffer.from('2').equals(req.executorHash));
    t.equal(req.taskKey, '3');
    t.equal(application.decodeData(req.inputs).foo, 'bar');
    t.ok(isUUID.v4(req.tags[0]));
    spy.restore();
});
